//
//  Player.m
//  BladeHero
//
//  Created by Jeames Bone on 22/07/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "Player.h"
#import "GameObject.h"

@implementation Player

const int walkSpeed = 2;
const int dashSpeed = 5;

@synthesize view, speed, target, dashUp;

- (id)init
{
    self = [super init];
    if (self) 
    {
        position = CGPointMake(100, 100);
        target = CGPointMake(position.x, position.y);
        size = CGSizeMake(32, 32);
        bounds = [self updateBounds];
        view = [[PlayerView alloc] initWithFrame:bounds];
        speed = walkSpeed;
        dashUp = YES;
    }
    return self;
}

- (void)moveTowardsPoint:(CGPoint)point
{
    CGPoint difference = CGPointMake(point.x - position.x, point.y - position.y);
    float xNormalized = (point.x - position.x) / fabs(sqrt(difference.x * difference.x + difference.y * difference.y));
    float yNormalized = (point.y - position.y) / fabs(sqrt(difference.x * difference.x + difference.y * difference.y));
    position.x += xNormalized * speed;
    position.y += yNormalized * speed;
    bounds = [self updateBounds];
    [view setFrame:bounds];
}

- (void)update
{
    if (target.x != position.x || target.y != position.y) 
    {
        [self moveTowardsPoint:target];
    }
    
    if (fabs(position.x - target.x) < 1.5)
    {
        target.x = position.x;
    }
    
    if (fabs(position.y - target.y) < 1.5)
    {
        target.y = position.y;
    }
}

- (void)stopMoving
{
    target.x = position.x;
    target.y = position.y;
}

- (void)dash
{
    if (dashUp)
    {
        speed = dashSpeed;
        [NSTimer scheduledTimerWithTimeInterval:1.0 
                                         target:self 
                                       selector:@selector(stopDashing:)
                                       userInfo:nil
                                        repeats:NO]; 
    }
}

- (void)stopDashing:(NSTimer *)timer
{
    NSLog(@"Dash over");
    speed = walkSpeed;
    dashUp = NO;
    [NSTimer scheduledTimerWithTimeInterval:5.0 
                                     target:self 
                                   selector:@selector(resetDash:)
                                   userInfo:nil
                                    repeats:NO]; 
}

- (void)resetDash:(NSTimer *)timer
{
    NSLog(@"Dash cooldown over");
    dashUp = YES;
}


@end
