//
//  TrialViewController.m
//  BladeHero
//
//  Created by Jeames Bone on 22/07/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "TrialViewController.h"
#import "Tile.h"
#import "TrainingDummy.h"

#import <QuartzCore/QuartzCore.h>

@interface TrialViewController ()

@end

@implementation TrialViewController

@synthesize player, displayLink, tiles, td;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    player = [Player new];
    [self.view addSubview:player.view];
    
    td = [TrainingDummy new];
    [self.view addSubview:td.view];
    
    tiles = [NSMutableArray arrayWithObjects: [[Tile alloc] initWithPosition:CGPointMake(0, 32) andImage:[UIImage imageNamed:@"tile1.png"] isSolid:YES],
                                              [[Tile alloc] initWithPosition:CGPointMake(0, 96) andImage:[UIImage imageNamed:@"tile1.png"] isSolid:YES],
                                              [[Tile alloc] initWithPosition:CGPointMake(0, 160) andImage:[UIImage imageNamed:@"tile1.png"] isSolid:YES],
                                              [[Tile alloc] initWithPosition:CGPointMake(0, 224) andImage:[UIImage imageNamed:@"tile1.png"] isSolid:YES], 
                                              [[Tile alloc] initWithPosition:CGPointMake(64, 160) andImage:[UIImage imageNamed:@"tile1.png"] isSolid:YES],
                                              [[Tile alloc] initWithPosition:CGPointMake(128, 224) andImage:[UIImage imageNamed:@"tile1.png"] isSolid:YES],nil];
    
    for (Tile *t in tiles) 
    {
        [self.view addSubview:t.view];
    }
    
    // Set up game loop
    displayLink = [CADisplayLink displayLinkWithTarget:self selector:@selector(update:)];
    [displayLink addToRunLoop:[NSRunLoop mainRunLoop] forMode:NSRunLoopCommonModes];
}

- (void) update:(CADisplayLink*)displayLink 
{
    for (Tile *t in tiles) 
    {
        if ([self collideBox:t withPlayer:player]) {
            t.view.alpha = 1;
        }
        else {
            t.view.alpha = 0.5;
        }
    }
    
    if ([self collideCircle:td withPlayer:player])
    {
        td.view.alpha = 0.5;
    }
    else
    {
        td.view.alpha = 1;
    }
    
    [player update];
}

- (BOOL) collideCircle:(GameObject *)object withPlayer:(Player *)p
{
    float objectRadius = player.size.width / 2;
    float playerRadius = player.size.width / 2;
    
    float distanceX = player.position.x - object.position.x;
    float distanceY = player.position.y - object.position.y;
    
    float radius = objectRadius + playerRadius;
    
    if ((distanceX * distanceX) + (distanceY * distanceY) < radius * radius)
    {
        float distance = sqrt(distanceX * distanceX + distanceY * distanceY);
        
        float normalisedX = distanceX / distance;
        float normalisedY = distanceY / distance;
        
        player.position = CGPointMake(object.position.x + normalisedX * radius, object.position.y + normalisedY * radius);
        
        return YES;
    }
    else
    {
        return NO;
    }
}

- (BOOL) collideBox:(GameObject *)object withPlayer:(Player *)p
{
    float radius = player.size.width / 2;
    
    // Find the closest point to the circle within the rectangle
    float closestX = [self clampValue:player.position.x min:object.position.x - object.size.width / 2 
                                                        max:object.position.x + object.size.width / 2];
    float closestY = [self clampValue:player.position.y min:object.position.y - object.size.height / 2 
                                                        max:object.position.y + object.size.height / 2];
    
    // Calculate the distance between the circle's center and this closest point
    float distanceX = player.position.x - closestX;
    float distanceY = player.position.y - closestY;
    
    // If the distance is less than the circle's radius, an intersection occurs
    float distanceSquared = (distanceX * distanceX) + (distanceY * distanceY);
    
    if (distanceSquared < (radius * radius))
    {
        // Resolve Collision
        
        float halfRectWidth = object.size.width * 0.5;
        float halfRectHeight = object.size.height * 0.5;
        
        float pushVectorX, pushVectorY;

        float relativePositionX = player.position.x - object.position.x;
        float relativePositionY = player.position.y - object.position.y;
        
        if(abs(relativePositionX) > halfRectWidth && abs(relativePositionY) > halfRectHeight) // Colliding with a corner
        {
            pushVectorX = relativePositionX;
            pushVectorY = relativePositionY;
            
            float edgeX, edgeY;
            
            if (relativePositionX > 0) 
                edgeX = halfRectWidth; 
            else 
                edgeX = -halfRectWidth;
            
            if (relativePositionY > 0) 
                edgeY = halfRectHeight; 
            else 
                edgeY = -halfRectHeight;   
            
            pushVectorX -= edgeX;
            pushVectorY -= edgeY;
            
            // We now have the vector from the corner of the rect to the point.
            float deltaLength = sqrt(pushVectorX * pushVectorX + pushVectorY * pushVectorY);
            float diff = radius - deltaLength; // Find out how far away we are from our ideal distance
            
            // Normalise the vector
            pushVectorX /= deltaLength;
            pushVectorY /= deltaLength;
            pushVectorX *= diff; // Now multiply it by the difference
            pushVectorY *= diff;
        }
        else // Colliding with an edge
        {
            if(relativePositionX > 0) // Ball is to the right
                pushVectorX = (halfRectWidth + radius) - relativePositionX;
            else
                pushVectorX = -((halfRectHeight + radius) + relativePositionX);
            
            if(relativePositionY > 0) // Ball is above
                pushVectorY = (halfRectHeight + radius) - relativePositionY;
            else
                pushVectorY = -((halfRectHeight + radius) + relativePositionY);
            
            if(abs(pushVectorX) < fabs(pushVectorY))
                pushVectorY = 0;
            else
                pushVectorX = 0;
        }
        
        player.position = CGPointMake(player.position.x + pushVectorX, player.position.y + pushVectorY);
        
        return YES;
    }
    else 
    {
        return NO;
    }
}

- (float)clampValue:(float)value min:(float)min max:(float)max
{
    value = MIN(value, max);
    value = MAX(value, min);
    
    return value;
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    UITouch *touch = [touches anyObject];
    CGPoint touchPoint = [touch locationInView:self.view];
    player.target = touchPoint;
    if (touch.tapCount == 2)
    {
        [player dash];
    }
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    [player stopMoving];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    UITouch *touch = [touches anyObject];
    CGPoint touchPoint = [touch locationInView:self.view];
    player.target = touchPoint;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
